package test.executor;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.BasicDiagnostic;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.impl.EPackageRegistryImpl;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.m2m.qvt.oml.BasicModelExtent;
import org.eclipse.m2m.qvt.oml.ExecutionContextImpl;
import org.eclipse.m2m.qvt.oml.ExecutionDiagnostic;
import org.eclipse.m2m.qvt.oml.ModelExtent;
import org.eclipse.m2m.qvt.oml.TransformationExecutor;
import org.eclipse.m2m.qvt.oml.util.Log;
import org.eclipse.m2m.qvt.oml.util.WriterLog;
import org.eclipse.papyrus.sysml.portandflows.PortandflowsPackage;
import org.eclipse.uml2.uml.UMLPackage;

public class SimpleUmlIncrementalModExecutor {

    public static void run(String path)
    {
        // Refer to an existing transformation via URI
        URI transformationURI = URI.createURI("platform:/plugin/test.m2m/transforms/SimpleUmlIncrementalMod.qvto");
        //
        // metamodels
        EPackage.Registry registry = new EPackageRegistryImpl(EPackage.Registry.INSTANCE);
        UMLPackage theUMLPackage = (UMLPackage)EPackage.Registry.INSTANCE.getEPackage(UMLPackage.eNS_URI);
        PortandflowsPackage thePortandflowsPackage=(PortandflowsPackage)EPackage.Registry.INSTANCE.getEPackage(PortandflowsPackage.eNS_URI);
        registry.put(theUMLPackage.getNsURI(), theUMLPackage);
        registry.put(thePortandflowsPackage.getNsURI(), thePortandflowsPackage);
        // create executor for the given transformation
        TransformationExecutor executor = new TransformationExecutor(transformationURI, registry);
        //
        // define the transformation inout
        // Remark: we take the objects from a resource, however
        // a list of arbitrary in-memory EObjects may be passed
        ResourceSet resourceSet = new ResourceSetImpl();
        EList<EObject> inOutObjects = resourceSet
                                .getResource(URI.createFileURI(path),true)
                                    .getContents();
        //
        // create the inout extent with its initial content
        ModelExtent inout = new BasicModelExtent(inOutObjects);
        //
        // setup the execution environment details -> 
        // configuration properties, logger, monitor object etc.
        OutputStreamWriter outStream = new OutputStreamWriter(System.out);
        Log log = new WriterLog(outStream);
        ExecutionContextImpl context = new ExecutionContextImpl();
        context.setConfigProperty("keepModeling", true);
        context.setLog(log);
        //
        // run the transformation assigned to the executor with the given 
        // input and output and execution context -> ChangeTheWorld(in, out)
        // Remark: variable arguments count is supported
        ExecutionDiagnostic result = executor.execute(context, inout);
        //
        // check the result for success
        if(result.getSeverity() == Diagnostic.OK)
        {
            // This list contains all the objects from the initial ModelExtent, plus all the ones
            // which were created during the QVTo transformations.
            List<EObject> inOutObjects2 = inout.getContents();
            resourceSet.getResource(URI.createFileURI(path),true)
                .getContents().addAll(inOutObjects2);
            try
            {
                resourceSet.getResource(URI.createFileURI(path),true)
                    .save(Collections.emptyMap());
            } catch (IOException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        else
        {
            // turn the result diagnostic into status and send it to error log
            IStatus status = BasicDiagnostic.toIStatus(result);
            Activator.getDefault().getLog().log(status);
        }

    }

}
